CAMERA = "Camera"
CAMERA_DATA_PATH = "camera_data_path"
CAMERA_1_NAME = "camera-40547499"
CAMERA_2_NAME = "camera-40579627"
CAMERA_3_NAME = "camera-40580655"

TESSERACT = "Tesseract"
IMAGE_DEMO = "image_demo"
OUTPUT_IMAGE_PATH = "output_image_path"
TMP_PATH = "tmp_path"
BENTOML_URL = "bentoml_url"
DEFAULT_CONF = 0.6
CONF_MINIMUM = "conf_minimum"

DEFAULT_CONFIG_FILE_NAME = "./config/cadivi_config.props"
